C This fortran fragment can be compiled and plugged into NumberCrunch II as
C an xCOD. It has the right form to be used as the appropriate 
C two-dimensional potential with xPROPOGATE.
C
C  The sCOD that tells NCII the arguments for this routine is
C     xCOD xVXY_NELSON(zN:num; xy:array), xy[12]=(x,y) on input, (dV/dx,dV/dy) on ouput, with V(x,y)=[y-0.5x^2]^2 + 0.05 x^2 
	subroutine VXY_NELSON(zN,xy)
	implicit EXTENDED (a-h,o-z), INTEGER (i-n)
	dimension xy(2)

C Vxy calculates the gradient of the Nelson potential. The
C potential itself is
C    V nelson (x,y) = [ y - 0.5 x^2 ]^2 + Zmu x^2/2
C where Zmu = 0.1
C 
C Therefore, the gradient is 
C   Vy = 2*(y - .5*x^2)
C   Vx = 2*(y - .5*x^2)*(-x) + Zmu*x = x*( Zmu - Vy ) 
C
C  On input, zN is 2 (the size of xy), and
C  the values for x and y are passed to the program in xy(1) and xy(2)
C  On ouput, those values are changed to Vx and Vy. 
	
	parameter (Zmu = 0.10) 

	x = xy(1)
	y = xy(2)
	
	Vy = 2.0*(Y - X*X/2.0)
	Vx = X*(Zmu - Vy)
	
	xy(1) = Vx
	xy(2) = Vy
	
	return
	end
	